"""
Generation of relatively unique IDs.
"""

import getopt
import md5
import os
import random
import socket
import string
import sys
import time
import types

class Hash:

	"""A base class which can be derived to handle new types of
	hashes; this is the same kind of interface that is used by the md5
	and sha modules (the moduels themselves act as hashers, see to
	below).  A hash is fed new data with the update method; the
	current state of the hash can be requested with the digest method,
	or in a text-printable version with hexdigest."""
	
	def __init__(self, data=None):
		"""Create a new hash; if data is presented, start with that."""
		if self.__class__ is Hash:
			raise NotImplemenetedError
		if data is not None:
			self.update(data)

	def update(self, data):
		"""Update the hash by sending it more information."""
		raise NotImplemenetedError

	def digest(self):
		"""Get the current digest of the hash in binary form."""
		raise NotImplemenetedError

	def hexdigest(self):
		"""Get the current digest, hopefully in a readable ASCII form."""
		raise NotImplemenetedError

class IdentityHash(Hash):

	"""The identity hash just displays the data it's been fed."""
	
	def __init__(self, data=None):
		Hash.__init__(self, data)
		self.buffer = ''

	def update(self, data):
		self.buffer += data

	def digest(self):
		return self.buffer

	def hexdigest(self):
		return self.buffer

	
class HashGenerator:

	"""A hash generator (or hasher) is a sort of factory for hashes;
	it will create a fresh hash object by calling it with no arguments
	(ideal if it is, say, a class).  Clients need only create a new
	Hasher object with the factory as the constructor, or derive from
	and override its class member HashFactory."""

	HashFactory = None

	def __init__(self, factory=None):
		"""Instantiate a new hasher object, with the optional factory."""
		if factory is not None:
			self.HashFactory = factory
	
	def new(self):
		"""Create a new instance of the hasher."""
		return self.HashFactory()


class UIDGenerator:

	"""The class which generates new UID hashes, via the generate
	method.  The constructor takes a extra argument that should be
	different for different instances created concurrently, and a
	hasher object; both are optional.  Finally, the updateCustom
	method can be overridden by subclasses to include special
	behavior."""

	Hasher = md5 # default hasher is md5
	
	def __init__(self, extra=None, hasher=None):
		"""Create a new generator.  If multiple generators will be in
		existence at the same time, it is best that they be given different
		extra arguments."""
		self.extra = extra
		if hasher is not None:
			self.Hasher = hasher
		self.index = 0
		self.dict = None

	def generate(self):
		"""Generate, create the state, process it, and return a hash object."""
		self.reset()
		self.update()
		return self.process()

	def process(self):
		"""After the state has been updated, build and return a hash object."""
		hash = self.Hasher.new()
		keys = self.dict.keys()
		keys.sort()
		for key in keys:
			datum = '%s %s\n' % (key, self.dict[key])
			hash.update(datum)
		return hash

	def reset(self):
		"""Reset the state."""
		self.dict = {}

	def set(self, key, value):
		"""Update one bit of information."""
		assert type(key) is types.StringType
		self.dict[key] = value

	def update(self):
		"""Update the state with current information."""
		self.updateSystem()
		self.updateHost()
		self.updateUser()
		self.updateEnvironment()
		self.updateProcess()
		self.updateLocals()
		self.updateCustom()
		self.updateEphemeral() # do this last

	def updateSystem(self):
		"""The operating system and the Python system."""
		self.set('version', string.split(sys.version, ' ', 1)[0])
		self.set('platform', sys.platform)
		self.set('executable', sys.executable)
		self.set('prefix', sys.prefix)
		self.set('pythonpath', string.join(sys.path, ':'))

	def updateHost(self):
		"""The current host."""
		self.set('uname', string.join(os.uname(), ':'))
		hostname = socket.getfqdn(socket.gethostname())
		self.set('hostname', hostname)
		self.set('ip', socket.gethostbyname(hostname))

	def updateUser(self):
		"""The current user, as he/she is executing this process."""
		self.set('login', os.getenv("USER"))
		self.set('uid', os.getuid())
		self.set('gid', os.getgid())
		self.set('euid', os.geteuid())
		self.set('egid', os.getegid())

	def updateEnvironment(self):
		"""The operating environment."""
		self.set('home', os.environ.get('HOME'))
		self.set('term', os.environ.get('TERM'))
		self.set('display', os.environ.get('DISPLAY'))
		self.set('path', os.environ.get('PATH'))
		self.set('termid', os.ctermid())
		self.set('cwd', os.getcwd())

	def updateProcess(self):
		"""The current process."""
		self.set('pid', os.getpid())
		self.set('pgrp', os.getpgrp())
		self.set('ppid', os.getppid())

	def updateLocals(self):
		"""Local information."""
		if self.extra is not None:
			self.set('extra', str(self.extra))
		self.set('index', self.index)
		self.index = self.index + 1

	def updateCustom(self):
		"""Custom information -- override to extend."""
		pass

	def updateEphemeral(self):
		"""Ephemeral information to throw into the mix."""
		__id = id(self)
		if __id < 0 : __id = id(self) * -1
		self.set('id', hex(__id))
		self.set('time', '%.17f' % time.time())
		self.set('random', '%.17f' % random.random())
		self.set('times', string.join(map(repr, os.times()), ':')) # very last

def get () :
	generator = UIDGenerator(None, None)
	hash = generator.generate()
	return hash.hexdigest()

__program__ = 'uid'
__version__ = '1.0.2'
__author__ = 'Erik Max Francis <max@alcyone.com>'
__copyright__ = 'Copyright (C) 2002 Erik Max Francis'
__license__ = 'GPL'

